/**
 * COMBASE KORONA.pos Client - customerdisplay.js - JavaScript Version 1.0.0
 */

// iframe references
var iframe;
var iframeDomain;


function DisplayRenderer() {
    var self = this;

    self.request = {};
    self.headerTemplate = $.templates('#headerTemplate');
    self.saleItemTemplate = $.templates('#saleItemTemplate');
    self.footerTemplate = $.templates('#footerTemplate');
    self.mediaTemplate = $.templates('#mediaTemplate');
    self.qrTemplate = $.templates('#qrTemplate');

    self.qrCodeGenerator;

    self.render = function render(request) {
        self.request = request;
        self.applyConfiguration();
        self.applyDisplayStatus();
    };

    self.applyConfiguration = function applyConfiguration() {
        if (self.request == null || self.request.customerDisplayConfiguration == null)
            return;

        var config = self.request.customerDisplayConfiguration;

        if (self.oldConfig && JSON.stringify(config) === JSON.stringify(self.oldConfig))
            return;

        self.oldConfig = config;

        if (config.fontScale) {
            $('.main-layout').css('font-size', config.fontScale + 'vh');
        }

        if (config.layout) {
            switch (config.layout) {
                case 'MEDIA_LEFT': {
                    $('.main-layout').css('flex-direction', 'row-reverse');
                    $('.receipt').css('flex', '1 1 1vw');
                    $('.receipt-header').css('flex', '');
                    $('.receipt-content').css('display', 'flex');
                    $('.receipt-footer').css('display', 'flex');
                    $('.media').css('flex', '2 2 1vw');
                    $('.media').css('display', 'flex');
                    break;
                }
                case 'MEDIA_LEFT_HALF': {
                    $('.main-layout').css('flex-direction', 'row-reverse');
                    $('.receipt').css('flex', '1 1 1vw');
                    $('.receipt').css('display', 'flex');
                    $('.receipt-header').css('flex', '');
                    $('.receipt-content').css('display', 'flex');
                    $('.receipt-footer').css('display', 'flex');
                    $('.media').css('flex', '1 1 1vw');
                    $('.media').css('display', 'flex');
                    break;
                }
                case 'MEDIA_RIGHT': {
                    $('.main-layout').css('flex-direction', 'row');
                    $('.receipt').css('flex', '1 1 1vw');
                    $('.receipt').css('display', 'flex');
                    $('.receipt-header').css('flex', '');
                    $('.receipt-content').css('display', 'flex');
                    $('.receipt-footer').css('display', 'flex');
                    $('.media').css('flex', '2 2 1vw');
                    $('.media').css('display', 'flex');
                    break;
                }
                case 'MEDIA_RIGHT_HALF': {
                    $('.main-layout').css('flex-direction', 'row');
                    $('.receipt').css('flex', '1 1 1vw');
                    $('.receipt').css('display', 'flex');
                    $('.receipt-header').css('flex', '');
                    $('.receipt-content').css('display', 'flex');
                    $('.receipt-footer').css('display', 'flex');
                    $('.media').css('flex', '1 1 1vw');
                    $('.media').css('display', 'flex');
                    break;
                }
                case 'MEDIA_FULL': {
                    $('.main-layout').css('flex-direction', 'row');
                    $('.receipt').css('display', 'none');
                    $('.media').css('flex', '1 1 1vw');
                    $('.media').css('display', 'flex');
                    break;
                }
                case 'MEDIA_NONE': {
                    $('.main-layout').css('flex-direction', 'row');
                    $('.receipt').css('flex', '1 1 1vw');
                    $('.receipt').css('display', 'flex');
                    $('.receipt-header').css('flex', '');
                    $('.receipt-content').css('display', 'flex');
                    $('.receipt-footer').css('display', 'flex');
                    $('.media').css('display', 'none');
                    break;
                }
                case 'MEDIA_FULL_WITH_LINE_DISPLAY': {
                    $('.main-layout').css('flex-direction', 'column-reverse');
                    $('.receipt').css('flex', '1 1 1vh');
                    $('.receipt').css('display', 'flex');
                    $('.receipt-header').css('flex', '1 1 1vh');
                    $('.receipt-content').css('display', 'none');
                    $('.receipt-footer').css('display', 'none');
                    $('.media').css('flex', '5 5 1vh');
                    $('.media').css('display', 'flex');
                    break;
                }
                default:
                    break;
            }
        }

        if (config.colorTheme) {
            $('body').attr('class', '');
            $('body').addClass(config.colorTheme + '-theme');
        }

        self.welcomeText = config.welcomeText;
        self.closedText = config.closedText;
        self.bookingMediaUrl = config.bookingMediaUrl;
        self.welcomeMediaUrl = config.welcomeMediaUrl;
        self.closedMediaUrl = config.closedMediaUrl;
    };

    self.applyDisplayStatus = function applyDisplayStatus() {
        if (self.request == null || self.request.customerDisplay == null)
            return;

        var systemCurrency = self.request.systemCurrency;
        var display = self.request.customerDisplay;
        var receipt = self.request.receipt;
        var currency = (receipt && receipt.currency) ? receipt.currency : systemCurrency;

        switch (display.status) {
            case 'CLOSED': {
                $('.message-content').css('display', 'flex');
                $('.receipt-content-list').css('display', 'none');
                $('.receipt-header').html("");
                $('.receipt-footer').html("");
                self.renderHeader(display, currency);
                self.renderMessage((display.text) ? display.text : self.closedText);
                self.renderMediaUrl((display.mediaUrl) ? display.mediaUrl : self.closedMediaUrl);
                break;
            }
            case 'WELCOME': {
                $('.message-content').css('display', 'flex');
                $('.receipt-content-list').css('display', 'none');
                $('.receipt-header').html("");
                $('.receipt-footer').html("");
                self.renderHeader(display, currency);
                self.renderMessage((display.text) ? display.text : self.welcomeText);
                self.renderMediaUrl((display.mediaUrl) ? display.mediaUrl : self.welcomeMediaUrl);
                break;
            }
            case 'BOOKING': {
                $('.message-content').css('display', 'none');
                $('.receipt-content-list').css('display', 'block');
                self.renderHeader(display, currency);
                self.renderSaleItems(receipt);
                self.renderFooter(display, receipt);
                if (display.digitalReceiptUrl)
                    self.renderDigitalReceipt(display);
                else
                    self.renderMediaUrl((display.mediaUrl) ? display.mediaUrl : self.bookingMediaUrl);
                break;
            }
            case 'TO_BE_PAID': {
                $('.message-content').css('display', 'none');
                $('.receipt-content-list').css('display', 'block');
                self.renderHeader(display, currency, receipt);
                self.renderSaleItems(receipt);
                self.renderFooter(display, receipt);
                if (display.digitalReceiptUrl)
                    self.renderDigitalReceipt(display);
                else
                    self.renderMediaUrl((display.mediaUrl) ? display.mediaUrl : self.bookingMediaUrl);
                break;
            }
            case 'CHANGE_OWED': {
                $('.message-content').css('display', 'none');
                $('.receipt-content-list').css('display', 'block');
                self.renderHeader(display, currency);
                self.renderSaleItems(receipt);
                self.renderFooter(display, receipt);
                self.renderMediaUrl((display.mediaUrl) ? display.mediaUrl : self.bookingMediaUrl);
                break;
            }
        }
    }

    self.renderHeader = function renderHeader(customerDisplay, currency, receipt) {
        var text = (customerDisplay.text) ? customerDisplay.text : '';
        var changeOwed = customerDisplay.changeOwed;
        var toBePaid = customerDisplay.toBePaid;
        var saleQuantity = customerDisplay.quantity;
        var salePrice = customerDisplay.price;
        var lines = [];

        switch (customerDisplay.status) {
            case 'CLOSED': {
                if ($('.receipt-content').css('display') == 'none') {
                    $('.receipt-header').css('justify-content', 'center');
                    lines.push(text);
                }
                break;
            }
            case 'WELCOME': {
                if ($('.receipt-content').css('display') == 'none') {
                    $('.receipt-header').css('justify-content', 'center');
                    lines.push(text);
                }
                break;
            }
            case 'BOOKING': {
                $('.receipt-header').css('justify-content', 'space-between');
                lines.push(((saleQuantity) ? saleQuantity : '') + ' x ' + text);
                lines.push(formatPrice(salePrice, currency));
                break;
            }
            case 'TO_BE_PAID': {
                $('.receipt-header').css('justify-content', 'space-between');
                if (receipt == null || receipt.sales == null || receipt.sales.length === 0)
                    break;
                lines.push(text);
                lines.push(formatPrice(toBePaid, currency));
                break;
            }
            case 'CHANGE_OWED': {
                $('.receipt-header').css('justify-content', 'space-between');
                lines.push(text);
                lines.push(formatPrice(changeOwed, currency));
                break;
            }
        }

        var renderlines = {
            line1: (lines.length > 0) ? lines[0] : '',
            line2: (lines.length > 0 && lines[1]) ? lines[1] : '',
        };

        $('.receipt-header').html(self.headerTemplate.render(renderlines));
    };

    self.renderMessage = function renderMessage(message) {
        $('.message-content').html(message);
    };

    self.renderSaleItems = function renderSaleItems(receipt) {
        if (receipt == null || receipt.sales == null || receipt.sales.length === 0) {
            $('.receipt-content-list').html("");
            return;
        }

        prepareReceiptSales(receipt);
        $('.receipt-content-list').html(self.saleItemTemplate.render(receipt));
        $('.receipt-content-list').scrollTop($('.receipt-content-list').prop("scrollHeight"));

    };

    self.renderFooter = function renderFooter(customerDisplay, receipt) {
        if (receipt == null) {
            $('.receipt-footer').html("");
            return;
        }

        var json = {
            receipt: receipt,
            toBePaidLabel: customerDisplay.toBePaidText
        };
        formatReceiptJson(json.receipt);
        $('.receipt-footer').html(self.footerTemplate.render(json));
    };

    self.renderDigitalReceipt = function renderDigitalReceipt(display) {
        var headerLine = display.digitalReceiptText1;
        var footerLine = display.digitalReceiptText2;

        // TODO extract texts out of API

        headerLine = headerLine.replace(/\n/g, '<br />');
        footerLine = footerLine.replace(/\n/g, '<br />');

        $('.media-content').html(self.qrTemplate.render({
            uri: display.digitalReceiptUrl,
            headerLine: headerLine,
            footerLine: footerLine,
            json: JSON.stringify(self.request, null, 4)
        }));
        $('#qrcode').qrcode({ width: 512, height: 512, text: display.digitalReceiptUrl });
        //force re-rendering media content for later
        self.oldMediaUrl = null;
    }

    self.renderMediaUrl = function renderMediaUrl(mediaUrl) {

        if (self.oldMediaUrl && mediaUrl === self.oldMediaUrl)
            return;
        self.oldMediaUrl = mediaUrl;

        if (mediaUrl == null) {
            $('.media-content').html("");
            return;
        }

        var type = 'html';

        if (self.isVideoUrl(mediaUrl)) {
            type = 'video';
        }
        else if (self.isImageUrl(mediaUrl) || mediaUrl.startsWith('masterdata:/images/')) {
            type = 'image';
        }

        if (type === 'html') {
            $.ajax({
                url: mediaUrl,
                dataType: 'text',
                success: function () {
                    $('.media-content').html(self.mediaTemplate.render({
                        type: type,
                        uri: mediaUrl,
                        json: JSON.stringify(self.request, null, 4)
                    }));
                    iframe = document.getElementById('childFrame').contentWindow;
                },
                error: function (jqXHR, textStatus, err) {
                    $('.media-content').html(self.mediaTemplate.render({
                        //type: 'error',
                        //error: 'Could not load url:',
                        type: type,
                        uri: mediaUrl,
                        json: JSON.stringify(self.request, null, 4)
                    }));
                    console.error(JSON.stringify(textStatus));
                    console.error(JSON.stringify(err));
                }
            });
        }
        else {
            //delete iframe and stop sending messages
            iframe = null;
            iframeDomain = null;
            $('.media-content').html(self.mediaTemplate.render({
                type: type,
                uri: mediaUrl,
                json: JSON.stringify(self.request, null, 4)
            }));
        }

        if (type == 'video') {
            var videoElem = $('#video');
            videoElem.bind('canplaythrough', function (event) {
                this.play();
            });
            videoElem.bind('ended', function (event) {
                this.currentTime = 0;
                this.play();
            });
        }

    };

    self.isImageUrl = function isImageUrl(url) {
        return (url.match(/\.(jpeg|jpg|gif|png)$/) != null);
    };

    self.isVideoUrl = function isVideoUrl(url) {
        return (url.match(/\.(mp4)$/) != null);
    };

    self.isBase64Image = function isBase64Image(str) {
        return str.startsWith('data:image/png;base64,');
    }
}

window.onload = function myOnLoadFunc() {
    // console.log('window.onload');

    var displayRenderer = new DisplayRenderer();

    korona_plugin_api.ready(function myReadyFunc() {
        // console.log('KORONA.plugin-api ready function triggered');
        // console.log('Rendering: ' + JSON.stringify(korona_plugin_api.request));
        displayRenderer.render(korona_plugin_api.request);
        if (iframe && iframe !== null && iframeDomain && iframeDomain !== null) {
            // console.log('Posting message to iframe');
            iframe.postMessage(JSON.stringify(korona_plugin_api.request), iframeDomain);
        }
    });

    if (korona_plugin_api.request === null) {
        // console.log('No POS found. initializing with demo');
        korona_plugin_api.initFromJson(getDemoData());
    }

    // console.log('Rendering demo data');
    displayRenderer.render(korona_plugin_api.request);

    // console.log('Register iframe messaging');
    window.addEventListener("message", function (e) {
        // console.log('childFrame sent: ' + e.data)
        if (e.data === 'loaded') {
            iframe = document.getElementById('childFrame').contentWindow;
            iframeDomain = $('#childFrame').prop('src');
            iframe.postMessage(JSON.stringify(korona_plugin_api.request), iframeDomain);
        }
        if (e.data === 'closed') {
            korona_plugin_api.closeCustomerDisplayCall();
        }
    }, false);
};

function formatReceiptJson(receipt) {
    if (!receipt)
        return;

    if (receipt.total) {
        receipt.total.value = formatPrice(receipt.total.value, receipt.currency);
        receipt.total.net = formatPrice(receipt.total.net, receipt.currency);
        receipt.total.gross = formatPrice(receipt.total.gross, receipt.currency);
        receipt.total.discount = formatPrice(receipt.total.discount, receipt.currency);
    }
    if (receipt.discounts && receipt.discounts.length > 0)
        receipt.discounts[0].amount = formatPrice(receipt.discounts[0].amount, receipt.currency);

}

function prepareReceiptSales(receipt) {
    receipt.sales.forEach(sale => {
        var dotIndex = sale.hierarchicalOutline.indexOf('.');
        var major = (dotIndex !== -1) ? sale.hierarchicalOutline.substring(0, dotIndex - 1) : sale.hierarchicalOutline;
        var minor = (dotIndex !== -1) ? sale.hierarchicalOutline.substring(dotIndex + 1, sale.hierarchicalOutline.length) : '';
        sale.saleType = (dotIndex !== -1) ? 'sale-child' : 'sale-parent';
        sale.saleSeparator = (major !== '1' && (minor === '' || minor === '0')) ? 'sale-separator' : '';

        sale.price = formatPrice(sale.receiptRepresentation.total.value, receipt.currency);
        sale.total.value = formatPrice(sale.receiptRepresentation.total.value, receipt.currency);
        if (sale.receiptRepresentation.total.discount.amount)
            sale.receiptRepresentation.total.discount.amount = formatPrice(sale.receiptRepresentation.total.discount.amount, receipt.currency);
        if (sale.receiptRepresentation.discounts && sale.receiptRepresentation.discounts.length > 0)
            sale.receiptRepresentation.discounts.forEach(discount => {
                discount.amount = formatPrice(discount.amount, receipt.currency);
                discount.saleType = sale.saleType;
            });

    });
}

function formatPrice(price, currency) {
    if (!price)
        price = 0.0;

    return price.toLocaleString(navigator.language, { minimumFractionDigits: 2, style: 'currency', currency: currency.isoCode.toLowerCase() });
}
